using System.Collections.Generic;
using UnityEditor;
using UnityEngine;

[SelectionBase]
public class HallPart : MonoBehaviour
{
    
    [SerializeField] GameObject _closed;
    [SerializeField] GameObject _window;
    [SerializeField] GameObject _open;
    [SerializeField] GameObject _smallOpen;
    [SerializeField] OpeningMode _openingMode;

    [MenuItem("Tools/Randomize All &g")]
    public static void RandomizeAll()
    {
        foreach (var hallpart in FindObjectsOfType<HallPart>())
        {
            var parentSnap = hallpart.GetComponentInParent<SnapPoint>();
            if (parentSnap != null && parentSnap.SnapTarget != null)
                hallpart.SetMode(OpeningMode.Open);
            else
                hallpart.RandomMode();
        }
    }

    void RandomMode()
    {
        var allModes = new List<OpeningMode>() {OpeningMode.Closed, OpeningMode.Window};
        if (name.Contains("Hall"))
        {
            allModes.Add(OpeningMode.SmallOpen);
            allModes.Add(OpeningMode.Open);
        }

        var mode =UnityEngine.Random.Range(0, allModes.Count);
        SetMode(allModes[mode]);
    }

    public void SetMode(OpeningMode mode)
    {
        _openingMode = mode;
        UnityEditor.EditorUtility.SetDirty(this);
        UnityEditor.EditorUtility.SetDirty(this.gameObject);
        PrefabUtility.RecordPrefabInstancePropertyModifications(this);
        UpdateModels();
    }
    
    void Update() => UpdateModels();

    void OnValidate() => UpdateModels();

    void UpdateModels()
    {
        if (_closed != null)
            _closed.SetActive(_openingMode == OpeningMode.Closed);
        if (_window != null)
            _window.SetActive(_openingMode == OpeningMode.Window);
        if (_open != null)
            _open.SetActive(_openingMode == OpeningMode.Open);
        if (_smallOpen != null)
            _smallOpen.SetActive(_openingMode == OpeningMode.SmallOpen);
    }
}
public enum OpeningMode
{
    Closed,
    Window,
    Open,
    SmallOpen,
}