using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using UnityEngine;
using Random = UnityEngine.Random;

[SelectionBase]
public class Hallway : MonoBehaviour
{
   
   [SerializeField] HallPart[] _hallParts;
   [SerializeField] int _index;

   void OnValidate()
   {
      _hallParts = GetComponentsInChildren<HallPart>();
   }
   
   [ContextMenu("Snap Align")]
   public void SnapAlign()
   {
      var snaps = GetComponentsInChildren<SnapPoint>();
      foreach (var snap in snaps)
         snap.SnapTarget = null;
      
      snaps[0].transform.localPosition = Vector3.zero;
      for (var i = 1; i < snaps.Length - 1; i += 2)
      {
         SnapAlign(snaps, i);
      }
   }

   [ContextMenu("Snap Align Next")]
   public void SnapAlignNext()
   {
      var snaps = GetComponentsInChildren<SnapPoint>();
      SnapAlign(snaps, _index);
      _index += 2;
   }
   
   static void SnapAlign(SnapPoint[] snaps, int i)
   {
      var snap = snaps[i];
      var other = snaps[i + 1];
      other.AlignTo(snap);
      other.SnapTo(snap);
      Debug.Log($"Snapped {snap} to {other}");
   }


   [ContextMenu("Connect All")]
   public void ConnectAll()
   {
      var snaps = GetComponentsInChildren<SnapPoint>();
      foreach (var snap in snaps)
         snap.SnapTarget = null;
      foreach (var snap in snaps)
      {
         var other = snap.FindNearestOther(snaps);
         if (other == null)
            continue;
         
         other.SnapTarget = snap;
         snap.SnapTarget = other;
         Debug.Log($"Connected {snap} to {other}");
      }
   }

   [ContextMenu("Snap All")]
   public void SnapAll()
   {
      var snaps = GetComponentsInChildren<SnapPoint>();
      foreach (var snap in snaps)
         snap.SnapTarget = null;
      foreach (var snap in snaps)
      {
         var other = snap.FindNearestOther(snaps);
         other.SnapTarget = snap;
         snap.SnapTarget = other;
         other.SnapTo(snap);
         Debug.Log($"Snapped {snap} to {other}");
      }
   }
   
   [ContextMenu("All Windows")]
   public void AllWindows()
   {
      foreach (var part in _hallParts)
         part.SetMode(OpeningMode.Window);
   }
   
   [ContextMenu("All Closed")]
   public void AllClosed()
   {
      foreach (var part in _hallParts)
         part.SetMode(OpeningMode.Closed);
   }

   [ContextMenu("Randomize")]
   public void Randomize()
   {
      var allModes = ((OpeningMode[]) Enum.GetValues(typeof(OpeningMode))).ToList();
      foreach (var part in _hallParts)
      {
         var mode = allModes.OrderBy(t => Random.Range(int.MinValue, Int32.MaxValue)).First();
         part.SetMode(mode);
      }
   }
   
   
   [ContextMenu("Randomize No Windows")]
   public void RandomizeNoWindows()
   {
      var allModes = ((OpeningMode[]) Enum.GetValues(typeof(OpeningMode))).ToList();
      allModes.Remove(OpeningMode.Window);      
      foreach (var part in _hallParts)
      {
         var mode = allModes.OrderBy(t => Random.Range(int.MinValue, Int32.MaxValue)).First();
         part.SetMode(mode);
      }
   }
   
}
