using System;
using UnityEngine;
using UnityEngine.AI;
using Random = UnityEngine.Random;

public class Npc : MonoBehaviour
{
    [Range(0, 19)]
    [SerializeField] int _modelIndex;
    [SerializeField] Animator _animator;
    [SerializeField] NavMeshAgent _navmeshAgent;
    [SerializeField] Npc _leader;
    [SerializeField] bool _wander;

    void MoveToRandomSpawnPoint()
    {
        var spawnPoints = GameObject.FindGameObjectsWithTag("Spawn Point");
        var chosen = spawnPoints[Random.Range(0, spawnPoints.Length)];
        transform.position = chosen.transform.position;
    }

    void OnEnable()
    {
        var siblings = transform.parent.GetComponentsInChildren<Npc>();
        if (siblings[0] != this)
            _leader = siblings[0];
    }

    void OnValidate()
    {
        _navmeshAgent = GetComponent<NavMeshAgent>();
        //_modelIndex = Random.Range(0, transform.childCount);
        
        for (int i = 0; i < transform.childCount; i++)
        {
            var child = transform.GetChild(i);
            child.gameObject.SetActive(i == _modelIndex);
            _animator = child.GetComponent<Animator>();
        }
    }

    void Update()
    {
        if (ShouldChooseDestination())
            ChooseDestination();
    }

    bool ShouldChooseDestination()
    {
        if (_wander == false)
            return false;
        return _navmeshAgent.pathStatus != NavMeshPathStatus.PathComplete || _navmeshAgent.remainingDistance <= 0f || _leader != null;
    }

    void ChooseDestination()
    {
        if (_leader != null)
        {
            var offset = _leader.transform.TransformDirection(Vector3.right);
            _navmeshAgent.SetDestination(_leader.Destination + offset);
        }
        else
        {
            Vector3 destination = SnappableRoom.ChooseRandom().transform.position;
            _navmeshAgent.SetDestination(destination);
        }
    }

    public Vector3 Destination => _navmeshAgent.destination;
}