using System.Collections.Generic;
using System.Linq;
using UnityEditor;
using UnityEngine;

[ExecuteInEditMode]
public class SnappableRoom : MonoBehaviour
{
    [SerializeField] public bool LockInPlace;
    [SerializeField] public SnapPoint[] Points;
    [SerializeField] Vector3 _lastPosition;

    [MenuItem("Tools/SnapRoom &k")]
    public static void SnapRoom()
    {
        var points = Selection.activeGameObject.GetComponentsInChildren<SnapPoint>()
            .Where(t => t.SnapTarget != null)
            .FirstOrDefault();
        points.SnapAndAlign();
    }

    [ContextMenu("Clear Snaps")]
    public void ClearSnaps()
    {
        foreach (var snap in Points)
        {
            if (snap.SnapTarget != null)
            {
                snap.SnapTarget.SnapTarget = null;
                snap.SnapTarget = null;
            }
        }
    }

    void OnValidate() => Points = GetComponentsInChildren<SnapPoint>();

    void Update()
    {
        if (LockInPlace && Vector3.Distance(transform.position, _lastPosition) > 0.1f)
        {
            _lastPosition = transform.position;
            var alreadyMoved = new HashSet<SnappableRoom>();
            MoveChildren(alreadyMoved);
        }
    }

    void MoveChildren(HashSet<SnappableRoom> alreadyMoved)
    {
        alreadyMoved.Add(this);
        foreach (var snapPoint in Points)
        {
            if (snapPoint.SnapTarget != null)
            {
                snapPoint.SnapTarget.SnapAndAlign();
                if (alreadyMoved.Contains(snapPoint.SnapTarget._snappableRoom) == false)
                    snapPoint.SnapTarget._snappableRoom.MoveChildren(alreadyMoved);
            }
        }
    }
    
    public static SnappableRoom SpawnRoom(string name)
    {
        var settings = Resources.Load<SpaceStationSettings>("Settings");
        var prefab = settings.SnappableRooms.FirstOrDefault(t => t.name == name);

        var instance = PrefabUtility.InstantiatePrefab(prefab.gameObject);
        Debug.Log($"Instance {instance}");
        var room = ((GameObject) instance).GetComponent<SnappableRoom>();
        return room;
    }

    [ContextMenu("Snap Children")]
    void SnapChildren()
    {
        var allPoints = FindObjectsOfType<SnapPoint>();
        foreach (var snap in Points)
        {
            var nearest = snap.FindNearestOther(allPoints);
            if (nearest != null)
            {
                snap.SnapTarget = nearest;
                nearest.SnapTarget = snap;
            }
        }
    }

    public SnapPoint NextOpenPoint()
    {
        var point = Points.FirstOrDefault(t => t.SnapTarget == null);
        if (point == null)
            return null;
        
        return point;
    }

    public static SnappableRoom ChooseRandom()
    {
        var all = FindObjectsOfType<SnappableRoom>();
        var index = Random.Range(0, all.Length);
        return all[index];
    }
}